/*
 * time.c  Timer functions for cpe
 */

#include <linux/time.h>
#include <linux/timex.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/time.h>
#include <asm/arch/cpe/cpe.h>
#include <asm/arch/cpe_int.h>


#define MAX_TIMER   3

cpe_time_reg_t *TimerBase[] ={  0, 
                                (cpe_time_reg_t *)CPE_TIMER1_VA_BASE,
                                (cpe_time_reg_t *)CPE_TIMER2_VA_BASE,
                                (cpe_time_reg_t *)CPE_TIMER3_VA_BASE                                
                             };

unsigned int cpe_timer_enable(unsigned int timer)
{   
    cpe_time_ctrl_t *t=(cpe_time_ctrl_t *)(CPE_TIMER1_VA_BASE+TIMER_CR);

    if ((timer == 0) || (timer > MAX_TIMER))
		return 0;

    switch(timer)
    {
        case 1:
	    t->Tm1UpDown=0;	        
    	    t->Tm1En=1;
    	    t->Tm1OfEn=1;
        	break;    
        case 2:
	    t->Tm2UpDown=0;	                
    	    t->Tm2En=1;
    	    t->Tm2OfEn=1;
        	break; 
        case 3:
	    t->Tm3UpDown=0;	                
    	    t->Tm3En=1;
    	    t->Tm3OfEn=1;
    	    break; 
        
        default:
        	break;   
    }
    return 1;
}


/* This routine stops the specified timer hardware. */
unsigned int cpe_timer_disable(unsigned int timer)
{
    cpe_time_ctrl_t *t=(cpe_time_ctrl_t *)(CPE_TIMER1_VA_BASE + TIMER_CR);
    
    if ((timer == 0) || (timer > MAX_TIMER))
		return 0;

    switch(timer)
    {
        case 1:
    	    t->Tm1En=0;
    	    t->Tm1OfEn=0;
    	    break;    
        case 2:
    	    t->Tm2En=0;
    	    t->Tm2OfEn=0;
    	    break; 
        case 3:
    	    t->Tm3En=0;
    	    t->Tm3OfEn=0;
    	    break; 
        
        default:
    	break;
   
    }
    return 1;
}

void cpe_timer_set_counter(unsigned int timer, unsigned int value)
{
    volatile cpe_time_reg_t *t = TimerBase[timer];    
    t->TimerValue = value; 
}

void cpe_timer_set_reload(unsigned int timer, unsigned int value)
{
    volatile cpe_time_reg_t *t = TimerBase[timer];    
    t->TimerLoad = value;   
}

// --------------------------------------------------------------------
//	warning:
//		timer = 1, 2, 3
// --------------------------------------------------------------------
unsigned int cpe_timer_get_counter(unsigned int timer)
{
    volatile cpe_time_reg_t *t = TimerBase[timer];
    return t->TimerValue;
}


unsigned long cpe_gettimeoffset (void)
{
	return 0;
}


