#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include "rt_tool.h"		/* Header */

UINT ConvertRssiToSignalQuality(NDIS_802_11_RSSI RSSI)
{
	UINT signal_quality;
    if (RSSI >= -50)
        signal_quality = 100;
    else if (RSSI >= -80)    // between -50 ~ -80dbm
        signal_quality = (UINT)(24 + (RSSI + 80) * 2.6);   
    else if (RSSI >= -90)   // between -80 ~ -90dbm
        signal_quality = (UINT)((RSSI + 90) * 2.6);   
    else    // < -84 dbm
        signal_quality = 0;
 
    return signal_quality;
}

/******************************* MAIN ********************************/
int wep_key_id = 1;

#define BUFSIZE (1024*4)



int
RL_Enable_WLAN(int skfd, struct iwreq *wrq, int enable)
{
  int status;
  char buf[1024];

  if(enable != 0 && enable != 1)
	return -1;
  sprintf(buf, "%d", enable);
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = strlen(buf);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x08, wrq);
  return status;
}

int
RL_Remove_Key(int skfd, struct iwreq *wrq)
{
  int status;

  // remove keys
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x0e, wrq);
  return status;
}

int
RL_Set_SSID(int skfd, struct iwreq *wrq, char *strSSID)
{
  char buf[1024];
  int len;
  int status;

  // set ssid
  memset(buf, 0, sizeof(NDIS_802_11_SSID));
  len = strlen(strSSID);
  memcpy(buf, &len, sizeof(len));
  strcpy(buf+sizeof(len), strSSID);
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = sizeof(NDIS_802_11_SSID);
  wrq->u.data.flags = 0;
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x0b, wrq);
  return status;
}

int
RL_Set_Auth_Mode(int skfd,struct iwreq *wrq, int mode)
{
  char buf[1024];
  char *AuthMode[] = {"open", "shared", "wpapsk", "wpanone"};
  int status;

  if(mode < 0 || mode > 3)
	return -1;
  // set auth mode
  strcpy(buf, AuthMode[mode]); // open // shared // wpapsk // wpanone
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = strlen(buf);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x03, wrq);
  return status;
}

int
RL_Set_Encryption_Type(int skfd,struct iwreq *wrq, int type)
{
  char buf[1024];
  char *EncType[] = {"none", "wep", "tkip", "aes" };
  int status;

  if(type < 0 || type > 3)
	return -1;
  // set encryption type
  strcpy(buf, EncType[type]); // none // wep // tkip // aes
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = strlen(buf);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x04, wrq);
  return status;
}

int
RL_Set_Default_Key_ID(int skfd,struct iwreq *wrq, int id)
{
  char buf[1024];
  int status;

  if(id < 1 || id > 4)
	return -1;
  // set default Key ID
  sprintf(buf, "%d", id); // 1,2,3,4 
  wep_key_id = id;
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = strlen(buf);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x05, wrq);
  return status;
}

int
RL_Set_WEP_Key(int skfd,struct iwreq *wrq, char *key)
{
  char buf[1024];
  int len = strlen(key);
  int status;

  // WEP Key length descriptions:
  // 5: wep 40 ascii
  //10: wep 40 hex 
  //13: wep 104 ascii
  //26: wep 104 hex

  if(len != 5 && len != 10 && len != 13 && len != 26)
	return -1;
  strcpy(buf, key);
  wrq->u.data.flags = wep_key_id - 1;  // 0, 1, 2, 3
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = strlen(buf);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x06, wrq);
  return status;
}

int
RL_Set_PSK_Key(int skfd,struct iwreq *wrq, char *key)
{
  char buf[1024];
  int len = strlen(key);
  int status;

  if(len < 8 || len > 63)
	return -1;
  // set PSK Key
  strcpy(buf, key);
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = strlen(buf);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x07, wrq);
  return status;
}

int
RL_Connect_Status(int skfd,struct iwreq *wrq, int *cs)
{
  NDIS_MEDIA_STATE	ConnectStatus;
  int status;

  //  media connect
  wrq->u.data.pointer = (caddr_t)&ConnectStatus;
  wrq->u.data.length = sizeof(ConnectStatus);
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x0c, wrq);
  printf("Connect status=%d\n", ConnectStatus);
  *cs = (int)ConnectStatus;
  return status;
}

int
RL_Get_RSSI(int skfd,struct iwreq *wrq)
{
  NDIS_802_11_RSSI	rssi;
  int nSigQua;
  int status;

  // get rssi
  wrq->u.data.pointer = (caddr_t)&rssi;
  wrq->u.data.length = sizeof(rssi);
  wrq->u.data.flags = 1;
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x0d, wrq);
  nSigQua = ConvertRssiToSignalQuality(rssi);
  printf("signal quality=(%d%%)(%d) ", nSigQua, (int)rssi);
  return status;
}

int
RL_LIST_SCAN(int skfd,struct iwreq *wrq, char *buf)
{
  int status;

  // list scan
  strcpy(buf, "1");
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = 1;
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x09, wrq);
  if(status < 0)
	return status;
  sleep(2);
  // get site survey
  wrq->u.data.pointer = (caddr_t)buf;
  wrq->u.data.length = BUFSIZE;
  wrq->u.data.flags = 1;
  status = ioctl(skfd, SIOCIWFIRSTPRIV+0x0a, wrq);

  return status;
}
